@@ -3,7 +3,7 @@
  *  Here we test GiNaC's archiving system. */
 
 /*
- *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2017 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,9 +41,9 @@ unsigned exam_archive()
 	e = -42 * x * pow(y, sin(y*Catalan)) * dirac_ONE()
 	    * epsilon_tensor(idx(fail(), 3), idx(0, 3), idx(y/2, 3))
 	  + lorentz_g(
-	      varidx(lst(x, -11*y, acos(2*x).series(x==3-5*I, 3)) * color_ONE()
+	      varidx(lst{x, -11*y, acos(2*x).series(x==3-5*I, 3)} * color_ONE()
 	        * metric_tensor(varidx(log(cos(128.0/(x*y))), 5), varidx(2, 5)), zeta(3)),
-	      varidx(diag_matrix(lst(-1, Euler, atan(x/y==-15*I/17)))
+	      varidx(diag_matrix({-1, Euler, atan(x/y==-15*I/17)})
 	        * delta_tensor(idx(x, 2), idx(wild(7), 3)), zeta(3), true),
 	      true
 	    )
@@ -64,7 +64,7 @@ unsigned exam_archive()
 		std::ifstream fin("exam.gar", std::ios_base::binary);
 		fin >> ar;
 	}
-	f = ar.unarchive_ex(lst(x, y, mu, dim), "expr 1");
+	f = ar.unarchive_ex(lst{x, y, mu, dim}, "expr 1");
 
 	ex difference = (f - e).expand();
 	if (!difference.is_zero()) {
