@@ -3,7 +3,7 @@
  *  Here we test manipulations on GiNaC's indexed objects. */
 
 /*
- *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2017 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -244,18 +244,16 @@ static unsigned edyn_check()
 	symbol Bx("Bx"), By("By"), Bz("Bz");
 
 	// Lorentz transformation matrix (boost along x axis)
-	matrix L(4, 4);
-	L =       gamma, -beta*gamma, 0, 0,
-	    -beta*gamma,       gamma, 0, 0,
-	              0,           0, 1, 0,
-	              0,           0, 0, 1;
+	matrix L = {{      gamma, -beta*gamma, 0, 0},
+		    {-beta*gamma,       gamma, 0, 0},
+		    {          0,           0, 1, 0},
+		    {          0,           0, 0, 1}};
 
 	// Electromagnetic field tensor
-	matrix F(4, 4);
-	F =  0, -Ex, -Ey, -Ez,
-		Ex,   0, -Bz,  By,
-		Ey,  Bz,   0, -Bx,
-		Ez, -By,  Bx,   0;
+	matrix F = {{ 0, -Ex, -Ey, -Ez},
+		    {Ex,   0, -Bz,  By},
+		    {Ey,  Bz,   0, -Bx},
+		    {Ez, -By,  Bx,   0}};
 
 	// Indices
 	symbol s_mu("mu"), s_nu("nu"), s_rho("rho"), s_sigma("sigma");
@@ -267,12 +265,12 @@ static unsigned edyn_check()
 	      * indexed(F, rho, sigma)).simplify_indexed();
 
 	// Extract transformed electric and magnetic fields
-	ex Ex_p = e.subs(lst(mu == 1, nu == 0)).normal();
-	ex Ey_p = e.subs(lst(mu == 2, nu == 0)).normal();
-	ex Ez_p = e.subs(lst(mu == 3, nu == 0)).normal();
-	ex Bx_p = e.subs(lst(mu == 3, nu == 2)).normal();
-	ex By_p = e.subs(lst(mu == 1, nu == 3)).normal();
-	ex Bz_p = e.subs(lst(mu == 2, nu == 1)).normal();
+	ex Ex_p = e.subs(lst{mu == 1, nu == 0}).normal();
+	ex Ey_p = e.subs(lst{mu == 2, nu == 0}).normal();
+	ex Ez_p = e.subs(lst{mu == 3, nu == 0}).normal();
+	ex Bx_p = e.subs(lst{mu == 3, nu == 2}).normal();
+	ex By_p = e.subs(lst{mu == 1, nu == 3}).normal();
+	ex Bz_p = e.subs(lst{mu == 2, nu == 1}).normal();
 
 	// Check results
 	result += check_equal(Ex_p, Ex);
@@ -285,7 +283,7 @@ static unsigned edyn_check()
 	// Test 2: check energy density and Poynting vector of electromagnetic field
 
 	// Minkowski metric
-	ex eta = diag_matrix(lst(1, -1, -1, -1));
+	ex eta = diag_matrix(lst{1, -1, -1, -1});
 
 	// Covariant field tensor
 	ex F_mu_nu = (indexed(eta, mu.toggle_variance(), rho.toggle_variance())
@@ -294,16 +292,16 @@ static unsigned edyn_check()
 
 	// Energy-momentum tensor
 	ex T = (-indexed(eta, rho, sigma) * F_mu_nu.subs(s_nu == s_rho) 
-	        * F_mu_nu.subs(lst(s_mu == s_nu, s_nu == s_sigma))
+	        * F_mu_nu.subs(lst{s_mu == s_nu, s_nu == s_sigma})
 	      + indexed(eta, mu.toggle_variance(), nu.toggle_variance())
-	        * F_mu_nu.subs(lst(s_mu == s_rho, s_nu == s_sigma))
+	        * F_mu_nu.subs(lst{s_mu == s_rho, s_nu == s_sigma})
 	        * indexed(F, rho, sigma) / 4).simplify_indexed() / (4 * Pi);
 
 	// Extract energy density and Poynting vector
-	ex E = T.subs(lst(s_mu == 0, s_nu == 0)).normal();
-	ex Px = T.subs(lst(s_mu == 0, s_nu == 1));
-	ex Py = T.subs(lst(s_mu == 0, s_nu == 2)); 
-	ex Pz = T.subs(lst(s_mu == 0, s_nu == 3));
+	ex E = T.subs(lst{s_mu == 0, s_nu == 0}).normal();
+	ex Px = T.subs(lst{s_mu == 0, s_nu == 1});
+	ex Py = T.subs(lst{s_mu == 0, s_nu == 2});
+	ex Pz = T.subs(lst{s_mu == 0, s_nu == 3});
 
 	// Check results
 	result += check_equal(E, (Ex*Ex+Ey*Ey+Ez*Ez+Bx*Bx+By*By+Bz*Bz) / (8 * Pi));
@@ -382,7 +380,7 @@ static unsigned dummy_check()
 	// GiNaC 1.2.1 had a bug here because p.i*p.i -> (p.i)^2
 	e = indexed(p, i) * indexed(p, i) * indexed(p, j) + indexed(p, j);
 	ex fi = exprseq(e.get_free_indices());
-	if (!fi.is_equal(exprseq(j))) {
+	if (!fi.is_equal(exprseq{j})) {
 		clog << "get_free_indices(" << e << ") erroneously returned "
 		     << fi << " instead of (.j)" << endl;
 		++result;
