@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's indexed expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2017 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -61,53 +61,53 @@ indexed::indexed() : symtree(not_symmetric())
 // other constructors
 //////////
 
-indexed::indexed(const ex & b) : inherited(b), symtree(not_symmetric())
+indexed::indexed(const ex & b) : inherited{b}, symtree(not_symmetric())
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1) : inherited(b, i1), symtree(not_symmetric())
+indexed::indexed(const ex & b, const ex & i1) : inherited{b, i1}, symtree(not_symmetric())
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1, const ex & i2) : inherited(b, i1, i2), symtree(not_symmetric())
+indexed::indexed(const ex & b, const ex & i1, const ex & i2) : inherited{b, i1, i2}, symtree(not_symmetric())
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3) : inherited(b, i1, i2, i3), symtree(not_symmetric())
+indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3) : inherited{b, i1, i2, i3}, symtree(not_symmetric())
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited(b, i1, i2, i3, i4), symtree(not_symmetric())
+indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited{b, i1, i2, i3, i4}, symtree(not_symmetric())
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2) : inherited(b, i1, i2), symtree(symm)
+indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2) : inherited{b, i1, i2}, symtree(symm)
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2, const ex & i3) : inherited(b, i1, i2, i3), symtree(symm)
+indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2, const ex & i3) : inherited{b, i1, i2, i3}, symtree(symm)
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited(b, i1, i2, i3, i4), symtree(symm)
+indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited{b, i1, i2, i3, i4}, symtree(symm)
 {
 	validate();
 }
 
-indexed::indexed(const ex & b, const exvector & v) : inherited(b), symtree(not_symmetric())
+indexed::indexed(const ex & b, const exvector & v) : inherited{b}, symtree(not_symmetric())
 {
 	seq.insert(seq.end(), v.begin(), v.end());
 	validate();
 }
 
-indexed::indexed(const ex & b, const symmetry & symm, const exvector & v) : inherited(b), symtree(symm)
+indexed::indexed(const ex & b, const symmetry & symm, const exvector & v) : inherited{b}, symtree(symm)
 {
 	seq.insert(seq.end(), v.begin(), v.end());
 	validate();
@@ -117,11 +117,11 @@ indexed::indexed(const symmetry & symm, const exprseq & es) : inherited(es), sym
 {
 }
 
-indexed::indexed(const symmetry & symm, const exvector & v, bool discardable) : inherited(v, discardable), symtree(symm)
+indexed::indexed(const symmetry & symm, const exvector & v) : inherited(v), symtree(symm)
 {
 }
 
-indexed::indexed(const symmetry & symm, std::auto_ptr<exvector> vp) : inherited(vp), symtree(symm)
+indexed::indexed(const symmetry & symm, exvector && v) : inherited(std::move(v)), symtree(symm)
 {
 }
 
@@ -166,7 +166,7 @@ void indexed::printindices(const print_context & c, unsigned level) const
 {
 	if (seq.size() > 1) {
 
-		exvector::const_iterator it=seq.begin() + 1, itend = seq.end();
+		auto it = seq.begin() + 1, itend = seq.end();
 
 		if (is_a<print_latex>(c)) {
 
@@ -243,12 +243,6 @@ bool indexed::info(unsigned inf) const
 	return inherited::info(inf);
 }
 
-struct idx_is_not : public std::binary_function<ex, unsigned, bool> {
-	bool operator() (const ex & e, unsigned inf) const {
-		return !(ex_to<idx>(e).get_value().info(inf));
-	}
-};
-
 bool indexed::all_index_values_are(unsigned inf) const
 {
 	// No indices? Then no property can be fulfilled
@@ -256,7 +250,8 @@ bool indexed::all_index_values_are(unsigned inf) const
 		return false;
 
 	// Check all indices
-	return find_if(seq.begin() + 1, seq.end(), bind2nd(idx_is_not(), inf)) == seq.end();
+	return find_if(seq.begin() + 1, seq.end(),
+	               [inf](const ex & e) { return !(ex_to<idx>(e).get_value().info(inf)); }) == seq.end();
 }
 
 int indexed::compare_same_type(const basic & other) const
@@ -265,12 +260,8 @@ int indexed::compare_same_type(const basic & other) const
 	return inherited::compare_same_type(other);
 }
 
-ex indexed::eval(int level) const
+ex indexed::eval() const
 {
-	// First evaluate children, then we will end up here again
-	if (level > 1)
-		return indexed(ex_to<symmetry>(symtree), evalchildren(level));
-
 	const ex &base = seq[0];
 
 	// If the base object is 0, the whole object is 0
@@ -324,9 +315,9 @@ ex indexed::thiscontainer(const exvector & v) const
 	return indexed(ex_to<symmetry>(symtree), v);
 }
 
-ex indexed::thiscontainer(std::auto_ptr<exvector> vp) const
+ex indexed::thiscontainer(exvector && v) const
 {
-	return indexed(ex_to<symmetry>(symtree), vp);
+	return indexed(ex_to<symmetry>(symtree), std::move(v));
 }
 
 unsigned indexed::return_type() const
@@ -377,7 +368,7 @@ ex indexed::expand(unsigned options) const
 void indexed::validate() const
 {
 	GINAC_ASSERT(seq.size() > 0);
-	exvector::const_iterator it = seq.begin() + 1, itend = seq.end();
+	auto it = seq.begin() + 1, itend = seq.end();
 	while (it != itend) {
 		if (!is_a<idx>(*it))
 			throw(std::invalid_argument("indices of indexed object must be of type idx"));
@@ -403,7 +394,7 @@ ex indexed::derivative(const symbol & s) const
 // global functions
 //////////
 
-struct idx_is_equal_ignore_dim : public std::binary_function<ex, ex, bool> {
+struct idx_is_equal_ignore_dim {
 	bool operator() (const ex &lh, const ex &rh) const
 	{
 		if (lh.is_equal(rh))
@@ -454,7 +445,7 @@ exvector indexed::get_dummy_indices(const indexed & other) const
 
 bool indexed::has_dummy_index_for(const ex & i) const
 {
-	exvector::const_iterator it = seq.begin() + 1, itend = seq.end();
+	auto it = seq.begin() + 1, itend = seq.end();
 	while (it != itend) {
 		if (is_dummy_pair(*it, i))
 			return true;
@@ -515,7 +506,7 @@ exvector ncmul::get_free_indices() const
 	return free_indices;
 }
 
-struct is_summation_idx : public std::unary_function<ex, bool> {
+struct is_summation_idx {
 	bool operator()(const ex & e)
 	{
 		return is_dummy_pair(e, e);
@@ -532,8 +523,8 @@ exvector integral::get_free_indices() const
 template<class T> size_t number_of_type(const exvector&v)
 {
 	size_t number = 0;
-	for(exvector::const_iterator i=v.begin(); i!=v.end(); ++i)
-		if(is_exactly_a<T>(*i))
+	for (auto & it : v)
+		if (is_exactly_a<T>(it))
 			++number;
 	return number;
 }
@@ -561,9 +552,11 @@ template<class T> static ex rename_dummy_indices(const ex & e, exvector & global
 		// to the global set
 		size_t old_global_size = global_size;
 		int remaining = local_size - global_size;
-		exvector::const_iterator it = local_dummy_indices.begin(), itend = local_dummy_indices.end();
+		auto it = local_dummy_indices.begin(), itend = local_dummy_indices.end();
 		while (it != itend && remaining > 0) {
-			if (is_exactly_a<T>(*it) && find_if(global_dummy_indices.begin(), global_dummy_indices.end(), bind2nd(idx_is_equal_ignore_dim(), *it)) == global_dummy_indices.end()) {
+			if (is_exactly_a<T>(*it) &&
+			    find_if(global_dummy_indices.begin(), global_dummy_indices.end(),
+			            [it](const ex &lh) { return idx_is_equal_ignore_dim()(lh, *it); }) == global_dummy_indices.end()) {
 				global_dummy_indices.push_back(*it);
 				global_size++;
 				remaining--;
@@ -635,8 +628,7 @@ bool reposition_dummy_indices(ex & e, exvector & variant_dummy_indices, exvector
 		for (size_t j=i+1; j<e.nops(); ++j) {
 			if (is_dummy_pair(e.op(i), e.op(j))) {
 				local_var_dummies.push_back(e.op(i));
-				for (exvector::iterator k = variant_dummy_indices.begin();
-						k!=variant_dummy_indices.end(); ++k) {
+				for (auto k = variant_dummy_indices.begin(); k!=variant_dummy_indices.end(); ++k) {
 					if (e.op(i).op(0) == k->op(0)) {
 						variant_dummy_indices.erase(k);
 						break;
@@ -678,8 +670,7 @@ bool reposition_dummy_indices(ex & e, exvector & variant_dummy_indices, exvector
 
 	// If a dummy index is encountered for the first time in the
 	// product, pull it up, otherwise, pull it down
-	for (exvector::iterator it2 = seq.begin()+1, it2end = seq.end();
-			it2 != it2end; ++it2) {
+	for (auto it2 = seq.begin()+1, it2end = seq.end(); it2 != it2end; ++it2) {
 		if (!is_exactly_a<varidx>(*it2))
 			continue;
 
@@ -690,10 +681,10 @@ bool reposition_dummy_indices(ex & e, exvector & variant_dummy_indices, exvector
 					/*
 					 * N.B. we don't want to use
 					 *
-					 *  e = e.subs(lst(
+					 *  e = e.subs(lst{
 					 *  *it2 == ex_to<varidx>(*it2).toggle_variance(),
 					 *  ex_to<varidx>(*it2).toggle_variance() == *it2
-					 *  ), subs_options::no_pattern);
+					 *  }, subs_options::no_pattern);
 					 *
 					 * since this can trigger non-trivial repositioning of indices,
 					 * e.g. due to non-trivial symmetry properties of e, thus
@@ -728,7 +719,7 @@ next_index: ;
 }
 
 /* Ordering that only compares the base expressions of indexed objects. */
-struct ex_base_is_less : public std::binary_function<ex, ex, bool> {
+struct ex_base_is_less {
 	bool operator() (const ex &lh, const ex &rh) const
 	{
 		return (is_a<indexed>(lh) ? lh.op(0) : lh).compare(is_a<indexed>(rh) ? rh.op(0) : rh) < 0;
@@ -771,9 +762,9 @@ static void product_to_exvector(const ex & e, exvector & v, bool & non_commutati
 template<class T> ex idx_symmetrization(const ex& r,const exvector& local_dummy_indices)
 {	exvector dummy_syms;
 	dummy_syms.reserve(r.nops());
-	for (exvector::const_iterator it = local_dummy_indices.begin(); it != local_dummy_indices.end(); ++it)
-			if(is_exactly_a<T>(*it))
-				dummy_syms.push_back(it->op(0));
+	for (auto & it : local_dummy_indices)
+			if(is_exactly_a<T>(it))
+				dummy_syms.push_back(it.op(0));
 	if(dummy_syms.size() < 2)
 		return r;
 	ex q=symmetrize(r, dummy_syms);
@@ -876,7 +867,7 @@ contraction_done:
 					// the product
 					bool is_a_product = (is_exactly_a<mul>(*it1) || is_exactly_a<ncmul>(*it1)) &&
 					                    (is_exactly_a<mul>(*it2) || is_exactly_a<ncmul>(*it2));
-					ex r = (non_commutative ? ex(ncmul(v, true)) : ex(mul(v)));
+					ex r = (non_commutative ? ex(ncmul(std::move(v))) : ex(mul(std::move(v))));
 
 					// If new expression is a product we can call this function again,
 					// otherwise we need to pass argument to simplify_indexed() to be expanded
@@ -943,7 +934,7 @@ contraction_done:
 
 	ex r;
 	if (something_changed)
-		r = non_commutative ? ex(ncmul(v, true)) : ex(mul(v));
+		r = non_commutative ? ex(ncmul(std::move(v))) : ex(mul(std::move(v)));
 	else
 		r = e;
 
@@ -1131,9 +1122,9 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
 			const ex & term = sum.op(i);
 			exvector dummy_indices_of_term;
 			dummy_indices_of_term.reserve(dummy_indices.size());
-			for(exvector::iterator i=dummy_indices.begin(); i!=dummy_indices.end(); ++i)
-				if(hasindex(term,i->op(0)))
-					dummy_indices_of_term.push_back(*i);
+			for (auto & i : dummy_indices)
+				if (hasindex(term,i.op(0)))
+					dummy_indices_of_term.push_back(i);
 			ex term_symm = idx_symmetrization<idx>(term, dummy_indices_of_term);
 			term_symm = idx_symmetrization<varidx>(term_symm, dummy_indices_of_term);
 			term_symm = idx_symmetrization<spinidx>(term_symm, dummy_indices_of_term);
@@ -1149,7 +1140,7 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
 		std::vector<terminfo> terms_pass2;
 		for (std::vector<terminfo>::const_iterator i=terms.begin(); i!=terms.end(); ) {
 			size_t num = 1;
-			std::vector<terminfo>::const_iterator j = i + 1;
+			auto j = i + 1;
 			while (j != terms.end() && j->symm == i->symm) {
 				num++;
 				j++;
@@ -1164,13 +1155,13 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
 
 		// Chop the symmetrized terms into subterms
 		std::vector<symminfo> sy;
-		for (std::vector<terminfo>::const_iterator i=terms_pass2.begin(); i!=terms_pass2.end(); ++i) {
-			if (is_exactly_a<add>(i->symm)) {
-				size_t num = i->symm.nops();
+		for (auto & i : terms_pass2) {
+			if (is_exactly_a<add>(i.symm)) {
+				size_t num = i.symm.nops();
 				for (size_t j=0; j<num; j++)
-					sy.push_back(symminfo(i->symm.op(j), i->orig, num));
+					sy.push_back(symminfo(i.symm.op(j), i.orig, num));
 			} else
-				sy.push_back(symminfo(i->symm, i->orig, 1));
+				sy.push_back(symminfo(i.symm, i.orig, 1));
 		}
 
 		// Sort by symmetrized subterms
@@ -1179,10 +1170,10 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
 		// Combine equal symmetrized subterms
 		std::vector<symminfo> sy_pass2;
 		exvector result;
-		for (std::vector<symminfo>::const_iterator i=sy.begin(); i!=sy.end(); ) {
+		for (auto i=sy.begin(); i!=sy.end(); ) {
 
 			// Combine equal terms
-			std::vector<symminfo>::const_iterator j = i + 1;
+			auto j = i + 1;
 			if (j != sy.end() && j->symmterm == i->symmterm) {
 
 				// More than one term, collect the coefficients
@@ -1215,7 +1206,7 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
 
 				// How many symmetrized terms of this original term are left?
 				size_t num = 1;
-				std::vector<symminfo>::const_iterator j = i + 1;
+				auto j = i + 1;
 				while (j != sy_pass2.end() && j->orig == i->orig) {
 					num++;
 					j++;
@@ -1239,7 +1230,7 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
 		}
 
 		// Add all resulting terms
-		ex sum_symm = (new add(result))->setflag(status_flags::dynallocated);
+		ex sum_symm = dynallocate<add>(result);
 		if (sum_symm.is_zero())
 			free_indices.clear();
 		return sum_symm;
@@ -1367,9 +1358,9 @@ void scalar_products::add(const ex & v1, const ex & v2, const ex & dim, const ex
 void scalar_products::add_vectors(const lst & l, const ex & dim)
 {
 	// Add all possible pairs of products
-	for (lst::const_iterator it1 = l.begin(); it1 != l.end(); ++it1)
-		for (lst::const_iterator it2 = l.begin(); it2 != l.end(); ++it2)
-			add(*it1, *it2, *it1 * *it2);
+	for (auto & it1 : l)
+		for (auto & it2 : l)
+			add(it1, it2, it1 * it2);
 }
 
 void scalar_products::clear()
@@ -1392,13 +1383,11 @@ ex scalar_products::evaluate(const ex & v1, const ex & v2, const ex & dim) const
 void scalar_products::debugprint() const
 {
 	std::cerr << "map size=" << spm.size() << std::endl;
-	spmap::const_iterator i = spm.begin(), end = spm.end();
-	while (i != end) {
-		const spmapkey & k = i->first;
+	for (auto & it : spm) {
+		const spmapkey & k = it.first;
 		std::cerr << "item key=";
 		k.debugprint();
-		std::cerr << ", value=" << i->second << std::endl;
-		++i;
+		std::cerr << ", value=" << it.second << std::endl;
 	}
 }
 
@@ -1448,13 +1437,13 @@ exvector get_all_dummy_indices(const ex & e)
 	exvector p;
 	bool nc;
 	product_to_exvector(e, p, nc);
-	exvector::const_iterator ip = p.begin(), ipend = p.end();
+	auto ip = p.begin(), ipend = p.end();
 	exvector v, v1;
 	while (ip != ipend) {
 		if (is_a<indexed>(*ip)) {
 			v1 = ex_to<indexed>(*ip).get_dummy_indices();
 			v.insert(v.end(), v1.begin(), v1.end());
-			exvector::const_iterator ip1 = ip+1;
+			auto ip1 = ip + 1;
 			while (ip1 != ipend) {
 				if (is_a<indexed>(*ip1)) {
 					v1 = ex_to<indexed>(*ip).get_dummy_indices(ex_to<indexed>(*ip1));
@@ -1473,27 +1462,24 @@ lst rename_dummy_indices_uniquely(const exvector & va, const exvector & vb)
 	exvector common_indices;
 	set_intersection(va.begin(), va.end(), vb.begin(), vb.end(), std::back_insert_iterator<exvector>(common_indices), ex_is_less());
 	if (common_indices.empty()) {
-		return lst(lst(), lst());
+		return lst{lst{}, lst{}};
 	} else {
 		exvector new_indices, old_indices;
 		old_indices.reserve(2*common_indices.size());
 		new_indices.reserve(2*common_indices.size());
 		exvector::const_iterator ip = common_indices.begin(), ipend = common_indices.end();
 		while (ip != ipend) {
-			ex newsym=(new symbol)->setflag(status_flags::dynallocated);
+			ex newsym = dynallocate<symbol>();
 			ex newidx;
 			if(is_exactly_a<spinidx>(*ip))
-				newidx = (new spinidx(newsym, ex_to<spinidx>(*ip).get_dim(),
-						ex_to<spinidx>(*ip).is_covariant(),
-						ex_to<spinidx>(*ip).is_dotted()))
-					-> setflag(status_flags::dynallocated);
+				newidx = dynallocate<spinidx>(newsym, ex_to<spinidx>(*ip).get_dim(),
+				                              ex_to<spinidx>(*ip).is_covariant(),
+				                              ex_to<spinidx>(*ip).is_dotted());
 			else if (is_exactly_a<varidx>(*ip))
-				newidx = (new varidx(newsym, ex_to<varidx>(*ip).get_dim(),
-						ex_to<varidx>(*ip).is_covariant()))
-					-> setflag(status_flags::dynallocated);
+				newidx = dynallocate<varidx>(newsym, ex_to<varidx>(*ip).get_dim(),
+				                             ex_to<varidx>(*ip).is_covariant());
 			else
-				newidx = (new idx(newsym, ex_to<idx>(*ip).get_dim()))
-					-> setflag(status_flags::dynallocated);
+				newidx = dynallocate<idx>(newsym, ex_to<idx>(*ip).get_dim());
 			old_indices.push_back(*ip);
 			new_indices.push_back(newidx);
 			if(is_a<varidx>(*ip)) {
@@ -1502,7 +1488,7 @@ lst rename_dummy_indices_uniquely(const exvector & va, const exvector & vb)
 			}
 			++ip;
 		}
-		return lst(lst(old_indices.begin(), old_indices.end()), lst(new_indices.begin(), new_indices.end()));
+		return lst{lst(old_indices.begin(), old_indices.end()), lst(new_indices.begin(), new_indices.end())};
 	}
 }
 
@@ -1537,15 +1523,12 @@ ex rename_dummy_indices_uniquely(exvector & va, const ex & b, bool modify_va)
 			lst indices_subs = rename_dummy_indices_uniquely(va, vb);
 			if (indices_subs.op(0).nops() > 0) {
 				if (modify_va) {
-					for (lst::const_iterator i = ex_to<lst>(indices_subs.op(1)).begin(); i != ex_to<lst>(indices_subs.op(1)).end(); ++i)
-						va.push_back(*i);
+					for (auto & i : ex_to<lst>(indices_subs.op(1)))
+						va.push_back(i);
 					exvector uncommon_indices;
 					set_difference(vb.begin(), vb.end(), indices_subs.op(0).begin(), indices_subs.op(0).end(), std::back_insert_iterator<exvector>(uncommon_indices), ex_is_less());
-					exvector::const_iterator ip = uncommon_indices.begin(), ipend = uncommon_indices.end();
-					while (ip != ipend) {
-						va.push_back(*ip);
-						++ip;
-					}
+					for (auto & ip : uncommon_indices)
+						va.push_back(ip);
 					sort(va.begin(), va.end(), ex_is_less());
 				}
 				return b.subs(ex_to<lst>(indices_subs.op(0)), ex_to<lst>(indices_subs.op(1)), subs_options::no_pattern|subs_options::no_index_renaming);
@@ -1568,18 +1551,17 @@ ex expand_dummy_sum(const ex & e, bool subs_idx)
 		else
 			v = get_all_dummy_indices(e_expanded);
 		ex result = e_expanded;
-		for(exvector::const_iterator it=v.begin(); it!=v.end(); ++it) {
-			ex nu = *it;
+		for (const auto & nu : v) {
 			if (ex_to<idx>(nu).get_dim().info(info_flags::nonnegint)) {
 				int idim = ex_to<numeric>(ex_to<idx>(nu).get_dim()).to_int();
 				ex en = 0;
 				for (int i=0; i < idim; i++) {
 					if (subs_idx && is_a<varidx>(nu)) {
 						ex other = ex_to<varidx>(nu).toggle_variance();
-						en += result.subs(lst(
+						en += result.subs(lst{
 							nu == idx(i, idim),
 							other == idx(i, idim)
-						));
+						});
 					} else {
 						en += result.subs( nu.op(0) == i );
 					}
