@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's overloaded operators. */
 
 /*
- *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2017 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -247,32 +247,32 @@ const numeric operator--(numeric & lh, int)
 
 const relational operator==(const ex & lh, const ex & rh)
 {
-	return dynallocate<relational>(lh, rh, relational::equal);
+	return relational(lh, rh, relational::equal);
 }
 
 const relational operator!=(const ex & lh, const ex & rh)
 {
-	return dynallocate<relational>(lh, rh, relational::not_equal);
+	return relational(lh, rh, relational::not_equal);
 }
 
 const relational operator<(const ex & lh, const ex & rh)
 {
-	return dynallocate<relational>(lh, rh, relational::less);
+	return relational(lh, rh, relational::less);
 }
 
 const relational operator<=(const ex & lh, const ex & rh)
 {
-	return dynallocate<relational>(lh, rh, relational::less_or_equal);
+	return relational(lh, rh, relational::less_or_equal);
 }
 
 const relational operator>(const ex & lh, const ex & rh)
 {
-	return dynallocate<relational>(lh, rh, relational::greater);
+	return relational(lh, rh, relational::greater);
 }
 
 const relational operator>=(const ex & lh, const ex & rh)
 {
-	return dynallocate<relational>(lh, rh, relational::greater_or_equal);
+	return relational(lh, rh, relational::greater_or_equal);
 }
 
 // input/output stream operators and manipulators
